<?php

/*
 * Global settings for boards.
 */

class EveryBoard_Global_Settings {

    private $framework;
    private $custom_css_classes;
    private $list_widgets;


    /**
     * Settings constructor, adds actions and sets private member.s
     * @param string $file Path of file that called constructor.
     */
    public function __construct( $file ) {

        global $pagenow;

        $options                    = get_option( 'render_framework' );
        $this->framework            = isset( $options['framework'] ) ? $options['framework'] : '';
        $classes                    = get_option( 'board_custom_css_classes', [] );
        $this->custom_css_classes   = $classes;
        $this->list_widgets         = $this->get_list_widget_settings();

        if ( is_admin() ) {
            add_action( 'admin_menu', array( &$this, 'register_board_settings_submenu_page' ) );
            add_action( 'admin_init', array( &$this, 'register_board_global_settings' ) );

            if( $pagenow === 'edit.php' && isset( $_GET['page'] ) && $_GET['page'] === 'boardsettings' ) {
                add_action( 'admin_enqueue_scripts', array( &$this, 'eb_global_settings_enqueue_scripts' ) );
            }
        }

        // Ajax Actions
        add_action( 'wp_ajax_eb_global_settings_ajax_get_css_classes', array( &$this, 'eb_global_settings_ajax_get_css_classes' ) );
        add_action( 'wp_ajax_eb_global_settings_ajax_add_css_class', array( &$this, 'eb_global_settings_ajax_add_css_class' ) );
        add_action( 'wp_ajax_eb_global_settings_ajax_delete_css_class', array( &$this, 'eb_global_settings_ajax_delete_css_class' ) );
        add_action( 'wp_ajax_eb_global_settings_ajax_get_css_classes_import', array( &$this, 'eb_global_settings_ajax_get_css_classes_import' ) );

        if( isset( $file ) && $file !== '') {
            register_activation_hook( $file, array( &$this, 'eb_global_settings_add_defaults' ) );
        }
    }

    /**
     * Execute action enqueue scripts, includes needed scripts and stylesheets for this page.
     */
    public function eb_global_settings_enqueue_scripts() {
        wp_enqueue_style( 'widgetprefab-style', EVERYBOARD_BASE . 'assets/dist/css/boardsettings.min.css' );
        wp_enqueue_script( 'eb_global_settings_js', EVERYBOARD_BASE . 'assets/dist/js/boardsettings.min.ugly.js' );
        wp_localize_script('eb_global_settings_js', 'translation_boardsettings', array(
            'remove_class_confirm' => __( 'Are you sure you want to remove this class?', 'everyboard' ),
            'remove_class_failure' => __( 'Could not remove class, please try again or contact an administrator.', 'everyboard' ),
            'class_import_failure' => __( 'Import failed, please check the format and try again.', 'everyboard' ),
            'add_class_failure' => __( 'There was a problem adding the class, please try again.', 'everyboard' ),
            'class_name_not_valid' => __('CSS Class name not valid, please try again. ( Only alphanumeric characters and "-" and "_" are allowed ).', "everyboard")
        ));
    }

    /**
     * Registers the settings for boards.
     */
    public function register_board_global_settings() {
        register_setting( 'everyboard-global-settings', 'render_framework');
        register_setting( 'everyboard-global-settings', 'template_settings');
        register_setting( 'everyboard-global-settings', 'everylist_active');
        register_setting( 'everyboard-global-settings', 'oclist_active' );
    }

    /**
     * AJAX FUNCTION - Prints the existing custom css classes.
     */
    public function eb_global_settings_ajax_get_css_classes() {
        print json_encode( $this->eb_global_settings_get_css_classes() );
        die();
    }

    /**
     * AJAX FUNCTION - Adds a given CSS class to the settings.
     * @return string either "success" or "fail" depending on status.
     */
    public function eb_global_settings_ajax_add_css_class() {

        $css_name   = $_POST['css_name'];
        $css_class  = $_POST['css_class'];
        $class_board  = isset( $_POST['board_class'] ) ? $_POST['board_class'] : false;
        $class_row  = isset( $_POST['row_class'] ) ? $_POST['row_class'] : false;
        $class_col  = isset( $_POST['col_class'] ) ? $_POST['col_class'] : false;
        $class_art  = isset( $_POST['art_class'] ) ? $_POST['art_class'] : false;
        $class_wid  = isset( $_POST['wid_class'] ) ? $_POST['wid_class'] : false;
        $unique_id  = isset( $_POST['id'] ) ? $_POST['id'] : uniqid( 'class_', false );

        $class_arr      = array(
            'name'    => $css_name,
            'class'   => $css_class,
            'board'   => $class_board,
            'row'     => $class_row,
            'column'  => $class_col,
            'article' => $class_art,
            'widget'  => $class_wid,
            'id'      => $unique_id
        );

        $insert_array   = $this->eb_global_settings_get_css_classes();
        $insert_array[] = $class_arr;
        //$insert_array = array();

        if( update_option( 'board_custom_css_classes', $insert_array ) ) {
            $this->custom_css_classes = $insert_array;
            print $unique_id;
        }
        else {
            print 'false';
        }

        die();
    }

    public function eb_global_settings_ajax_delete_css_class() {

        if( isset( $_POST['classId'] ) ) {

            $classId    = $_POST['classId'];
            $del_array  = $this->eb_global_settings_get_css_classes();
            
            $del_array = array_filter($del_array, function($class) use ($classId) {
                return $class['id'] !== $classId;
            });

            if( update_option( 'board_custom_css_classes', array_values( $del_array ) ) ) {
                $this->custom_css_classes = $del_array;
                print 'success';
            }
            else {
                print 'false';
            }
        }
        else {
            print 'false';
        }

        die();
    }

    /**
     * Returns existing CSS classes.
     * @return array List of current classes name of class is key and the actual class is the value.
     */
    public function eb_global_settings_get_css_classes() {
        return $this->custom_css_classes;
    }

    /**
     * Registers menu item for settings page, a submenu page of "Boards".
     */
    public function register_board_settings_submenu_page() {
        add_submenu_page( 'edit.php?post_type=everyboard', 'Board Settings', __('Board Settings', 'everyboard'), 'manage_options', 'boardsettings', array( &$this, 'admin_page' ) );
    }

    /**
     * Adds default values for board settings, runs on activation of plugin.
     */
    public function eb_global_settings_add_defaults() {

        $eb_framework_helper = new EveryBoard_Framework_Helper( false );
        update_option( 'render_framework', [
            'framework' => $eb_framework_helper->eb_default_framework
        ] );
    }

    /**
     * Get a list of current CSS classes formatted for export / import.
     */
    public function eb_global_settings_ajax_get_css_classes_export() {

        $classes    = $this->eb_global_settings_get_css_classes();
        $ret_str    = '';

        foreach ($classes as $key => $value) {
            $ret_str .= $key . '|' . $value . "\n";
        }

        print $ret_str;

        die();
    }

    /**
     * Imports a list of CSS classes.
     */
    public function eb_global_settings_ajax_get_css_classes_import() {

        if( isset( $_POST['importData'] ) ) {

            try {

                $import_data    = json_decode( stripslashes( $_POST['importData'] ) );
                $insert_array   = $this->eb_global_settings_get_css_classes();

                //old to new import
                if( !isset( $import_data ) ) {
                    $import_data = $_POST['importData'];
                    $lines = array_filter( explode(PHP_EOL, $import_data) );


                    foreach( $lines as $item ) {
                        $itemparts = explode( '|', $item );

                        $class_arr = array(
                            'name'   => $itemparts[1],
                            'class'  => $itemparts[0],
                            'board'  => 'true',
                            'row'    => 'true',
                            'column' => 'true',
                            'article'=> 'true',
                            'widget' => 'true',
                            'id'     => uniqid( 'class_', false )
                        );

                        $insert_array[] = $class_arr;
                    }
                } else {
                    foreach ( $import_data as $item ) {

                        $class_arr      = array(
                            'name'    => $item->name,
                            'class'   => $item->class,
                            'board'   => $item->board,
                            'row'     => $item->row,
                            'column'  => $item->column,
                            'article' => $item->article,
                            'widget'  => $item->widget,
                            'id'      => uniqid( 'class_', false )
                        );

                        $insert_array[] = $class_arr;
                    }
                }

                if( update_option( 'board_custom_css_classes', $insert_array ) ) {
                    print __('success', 'everyboard');
                }
                else {
                    print __('false', 'everyboard');
                }
            }
            catch( Exception $e ) {
                print __('false', 'everyboard');
            }
        }
        else {
            print __('false', 'everyboard');
        }


        die();
    }

    /**
     * Renders the actual options page for board settings.
     */
    public function admin_page() {

        $eb_framework_helper    = new EveryBoard_Framework_Helper( false );
        $frameworks             = $eb_framework_helper->get_frameworks();
        $active_framework       = esc_attr( $this->framework );
        $templates              = EveryBoard_Settings::get_all_templates()?: [];
        $saved_templates        = EveryBoard_Settings::get_templates();
       
        # Start HTML ?>
        <div class="wrap">
            <h2><?php echo __('EveryBoard Settings', 'everyboard'); ?></h2>
                <?php settings_errors(); ?>
                <form method="post" action="options.php">
                <?php settings_fields( 'everyboard-global-settings' ); ?>
                <table class="form-table">
                    <tr valign="top">
                        <th colspan="2"><h3><?php echo __('Board Rendering Framework', 'everyboard'); ?></h3></th>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <label for="board_settings_framework_selector"><?php echo __('Rendering Framework:', 'everyboard'); ?></label>
                            <p class="description"><?php echo __('Choose the framework to use when rendering the board data. Normally this should be the same framework that your theme is built on (if any).', 'everyboard'); ?></p>
                        </th>
                        <td>
                            <select id="board_settings_framework_selector" name="render_framework[framework]">
                                <?php foreach ( $frameworks as $slug => $framework ) { ?>
                                    <option value="<?php echo $slug; ?>" <?php selected( $slug, $active_framework ); ?>><?php echo $framework; ?></option>
                                <?php } ?>
                            </select>
                        </td>
                    </tr>

                    <tr valign="top">
                        <th colspan="2"><h3><?php echo __('List widgets', 'everyboard'); ?></h3></th>
                    </tr>

                    <?php foreach($this->list_widgets as $key => $widget) { ?>
                        <?php $input_id = "board_settings_active_list_{$key}"; ?>
                        <tr valign="top">
                            <th scope="row">
                                <label for="<?php echo $input_id; ?>"><?php echo $widget[ 'name' ]; ?></label>
                                <p class="description"><?php echo $widget[ 'description' ]; ?></p>
                            </th>
                            <th>Active:<br/>
                                <input id="<?php echo $input_id; ?>" type="checkbox" style="margin: 10px;" value="true" name="<?php echo $key; ?>_active" <?php checked( $widget[ 'active' ] ); ?>/>
                            </th>
                        </tr>
                    <?php } ?>

                    <tr valign="top">
                        <th colspan="2"><h3><?php echo __('Board Custom CSS Classes', 'everyboard'); ?></h3></th>
                    </tr>

                    <tr valign="top" class="table_row_extra_bottom_space">

                        <th scope="row">
                            <label for="board_settings_css_classes_add_name_input"><?php echo __('Add board custom CSS class:', 'everyboard'); ?></label>
                            <p class="description"><?php echo __('Add a new CSS class that will be available when editing an board.', 'everyboard'); ?></p>
                        </th>

                        <td>
                            <label for="board_settings_css_classes_add_name_input"><?php echo __('Class Name', 'everyboard'); ?>
                                <span class="description">(<?php echo __('the exact name of the actual css class', 'everyboard'); ?>):</span></label><br/>
                            <input type="text" name="" value="" id="board_settings_css_classes_add_name_input"/><br/>

                            <label for="board_settings_css_classes_add_class_input"><?php echo __('Class Description', 'everyboard'); ?>
                                <span class="description">(<?php echo __('a description of the class for the end user', 'everyboard'); ?>):</span></label><br />
                            <input type="text" name="" value="" id="board_settings_css_classes_add_class_input" /><br />

                            <div id="class-type-container">
                                <?php echo __('Accessible to items of type:', 'everyboard'); ?>
                                <table class="class-type-table board-settings-table">
                                    <tr>
                                        <td><label for="board_settings_css_board_class"><?php echo __('Board:', 'everyboard'); ?></label></td>
                                        <td><input id="board_settings_css_board_class" type="checkbox" name="board-class" /></td>
                                    </tr>
                                    <tr>
                                        <td><label for="board_settings_css_row_class"><?php echo __('Row:', 'everyboard'); ?></label></td>
                                        <td><input id="board_settings_css_row_class" type="checkbox" name="row-class" /></td>
                                    </tr>
                                    <tr>
                                        <td><label for="board_settings_css_col_class"><?php echo __('Column:', 'everyboard'); ?></label></td>
                                        <td><input id="board_settings_css_col_class" type="checkbox" name="col-class" /></td>
                                    </tr>
                                    <tr>
                                        <td><label for="board_settings_css_art_class"><?php echo __('Article:', 'everyboard'); ?></label></td>
                                        <td><input id="board_settings_css_art_class" type="checkbox" name="art-class" /></td>
                                    </tr>
                                    <tr>
                                        <td><label for="board_settings_css_wid_class"><?php echo __('Widget:', 'everyboard'); ?></label></td>
                                        <td><input id="board_settings_css_wid_class" type="checkbox" name="wid-class" /></td>
                                    </tr>
                                </table>
                            </div>

                            <a href="#" class="button" id="board_settings_css_classes_add_button"><?php echo __('Add class', 'everyboard'); ?></a>
                        </td>
                    </tr>

                    <tr valign="top">
                        <th scope="row">
                            <label for="board_settings_css_classes_list"><?php echo __('Current board custom CSS classes:', 'everyboard'); ?></label>
                            <p class="description"><?php echo __('The CSS classes that are available to users when editing a board.', 'everyboard'); ?></p>
                        </th>
                        <td>
                            <table id="board_settings_css_classes_table" class="board-settings-table">
                                <tr>
                                    <th><?php echo __('Class name', 'everyboard'); ?></th>
                                    <th><?php echo __('Description', 'everyboard'); ?></th>
                                    <th><?php echo __('For types', 'everyboard'); ?></th>
                                </tr>
                            </table>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <label for="board_settings_css_classes_export_area"><?php echo __('Export / Import board custom CSS classes:', 'everyboard'); ?></label>
                            <p class="description"><?php echo __('Paste CSS classes from another site or copy the existing ones to export to another site.', 'everyboard'); ?></p>
                        </th>
                        <td>
                            <textarea class="board_settings_export_area" id="board_settings_css_classes_export_area" rows="10" cols="100"></textarea>
                            <p>
                                <a href="#" class="button" id="board_settings_css_classes_import_button"><?php echo __('Import CSS Classes', 'everyboard'); ?></a>
                                <a href="#" class="button" id="board_settings_css_classes_export_button"><?php echo __('Export CSS Classes', 'everyboard'); ?></a>
                            </p>
                        </td>
                    </tr>

                    <tr valign="top">

                        <th scope="row">
                            <label for="board_settings_framework_selector"><?php echo __('Templates:', 'everyboard'); ?></label>
                            <p class="description"><?php echo __('Tick the teaser layout templates you want to register for use in boards and widgets.', 'everyboard'); ?></p>
                        </th>

                        <td>
                            <table class="class-type-table board-settings-table">
                                <?php foreach ( $templates as $template => $temp ) { ?>
                                    <?php $filename = urlencode(pathinfo($temp, PATHINFO_FILENAME)) ?>
                                    <tr>
                                        <th>
                                            <label for="<?php echo "board_settings_template_{$filename}"; ?>" class="check-boxes"><?php echo $template ?></label>
                                        </th>
                                        <?php if( $temp === 'ocarticle-default.php' ) { ?>
                                            <td>
                                                <strong>Default Template</strong>
                                                <input type="hidden" value="<?php echo $temp ?>" name="template_settings[<?php echo $template; ?>]">
                                            </td>
                                        <?php } else { ?>
                                            <td>
                                                <input id="<?php echo "board_settings_template_{$filename}"; ?>" type="checkbox" <?php checked( in_array( $temp, $saved_templates, true ) ) ?> value="<?php echo $temp; ?>" name="template_settings[<?php echo $template; ?>]">
                                            </td>
                                        <?php } ?>
                                    </tr>
                                <?php } ?>
                            </table>
                        </td>

                    </tr>

                </table>
                <?php submit_button(); ?>
            </form>
        </div>
        <?php # End HTML
    }
    
    public function get_list_widget_settings() {
        $list_widgets = [
            'everylist' => [
                'name'        => 'EveryList',
                'description' => __( 'Internal lists in Wordpress, lets you structure lists of articles and use them in boards.', 'everyboard' ),
                'active'      => static::is_everylist_active()
            ],
            'oclist'    => [
                'name'        => 'OC List',
                'description' => __( 'External lists saved in Open Content, requires Open Content 2.1.', 'everyboard' ),
                'active'      => static::is_oclist_active()
            ]
        ];

        return $list_widgets;
    }

    public static function is_everylist_active() {
        return get_option('everylist_active', 'true') === 'true';
    }

    public static function is_oclist_active() {
        return get_option('oclist_active', 'true') === 'true';
    }
}
